#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <packet.h>
#include <preproc.h>
#include <decode.h>
#include <plugin.h>

/* Stuff we import */
proc_preproc_activate preproc_activate;
proc_preproc_dispatch preproc_dispatch;
proc_decode_proto decode_proto;
struct proto *tcpproto=NULL;

/* These are optionally implemented, so don't check em */
u_int8_t omask=(TCP_ECE|TCP_CWR);

/* Mask to determine fragments */
unsigned short ipfmask=__constant_htons(IP_MF|IP_OFFMASK);

/* Preprocessor entry point */
int tcpstream_process(struct packet *pkt, unsigned int i, void *priv)
{
	struct pkt_iphdr *iph;
	struct pkt_tcphdr *tcph;
	
	if ( i+1 >= pkt->llen ) return PREPROC_OK;
	if ( pkt->layer[i+1].proto != tcpproto ) return PREPROC_OK;

	iph=pkt->layer[i].h.ip;
	tcph=pkt->layer[i+1].h.tcp;

	/* Don't bother with fragments */
	if ( iph->frag_off & ipfmask ) return PREPROC_OK;

	return PREPROC_OK;
}

int tcpstream_init(char *args)
{
	if ( tcpproto ) {
		printf("tcpstream: Can't add tcpstream twice!\n");
		return 0;
	}

	if ( !(tcpproto=decode_proto("tcp")) ) {
		printf("tcpstream: tcp protocol not loaded!\n");
		return 0;
	}
	
	/* TCP stream reassembly registering for IP realm?
	 * Well, we are only actually interested in TCP in IP */
	if (preproc_activate("ip", tcpstream_process, NULL)) {
		printf("tcpstream: TCP stream reassembly ENABLED.\n");
		return 1;
	}else{
		return 0;
	}
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	proc_preproc_add preproc_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("preproc.tcpstream", "TCP stream reassembly");
	PLUGIN_VERSION(0, 1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@ecsc.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(preproc_add=in->import("preproc.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(preproc_activate=in->import("preproc.activate")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(preproc_dispatch=in->import("preproc.dispatch")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	if ( !(decode_proto=in->import("decode.proto")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=preproc_add("tcpstream", tcpstream_init);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
